/*  Copyright 2006 Jonas Minnberg

    This file is part of OldPlay - a portable, multiformat musicplayer.

    OldPlay is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OldPlay is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with OldPlay; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include <stdio.h>
#include <string.h>
#include <modplug.h>

#include "modplug.h"
#include "stdafx.h"
#include "sndfile.h"

//#include "../SDLScreen.h"

#include "../plugin.h"

static char *types[] = { "", "Protracker", "ScreamTracker", "FastTracker II", "OctaMed",
"MTM", "ImpulseTracker", "669 Composer", "UltraTracker", "STM", "Farandole", "Wave",
"Asylum/Advanced MF", "AMS", "DSM/DSIK", "DigiTracker", "Oktalyzer", "Midi",
"DMF/X-Tracker", "PolyTracker", "DigiBooster Pro", "MadTracker 2.0", "Asylum/Advanced MF0", 
"PSM", "J2B" };



#define MOD_TYPE_NONE		0x00
#define MOD_TYPE_MOD		0x01
#define MOD_TYPE_S3M		0x02
#define MOD_TYPE_XM			0x04
#define MOD_TYPE_MED		0x08
#define MOD_TYPE_MTM		0x10
#define MOD_TYPE_IT			0x20
#define MOD_TYPE_669		0x40
#define MOD_TYPE_ULT		0x80
#define MOD_TYPE_STM		0x100
#define MOD_TYPE_FAR		0x200
#define MOD_TYPE_WAV		0x400
#define MOD_TYPE_AMF		0x800
#define MOD_TYPE_AMS		0x1000
#define MOD_TYPE_DSM		0x2000
#define MOD_TYPE_MDL		0x4000
#define MOD_TYPE_OKT		0x8000
#define MOD_TYPE_MID		0x10000
#define MOD_TYPE_DMF		0x20000
#define MOD_TYPE_PTM		0x40000
#define MOD_TYPE_DBM		0x80000
#define MOD_TYPE_MT2		0x100000
#define MOD_TYPE_AMF0		0x200000
#define MOD_TYPE_PSM		0x400000
#define MOD_TYPE_J2B		0x800000
#define MOD_TYPE_UMX		0x80000000 // Fake type



#ifndef WIN32
#define stricmp strcasecmp
#endif

static struct sound_plugin plugin;

static ModPlugFile *mpfile = NULL;

struct _ModPlugFile
{
	CSoundFile mSoundFile;
};

static int total_samples = 0;
static int song_end = 99999;

static int init_data(char *name, void *data, int size)
{
	if(mpfile)
	    ModPlug_Unload(mpfile);

	ModPlug_Settings settings;
    ModPlug_GetSettings(&settings);

	settings.mResamplingMode = MODPLUG_RESAMPLE_LINEAR; /* RESAMP */
	settings.mChannels = 2;
	settings.mBits = 16;
#ifdef WIN32
	settings.mFlags = MODPLUG_ENABLE_OVERSAMPLING;
#else
	settings.mFlags = 0;
#endif
	settings.mFrequency = plugin.freq;
	settings.mFlags = 0;
    /* insert more setting changes here */
	ModPlug_SetSettings(&settings);

	mpfile = ModPlug_Load(data, size);

	int t = ((struct _ModPlugFile*)mpfile)->mSoundFile.GetType();
	int mt = 0;
	while(t)
	{
		t = t>>1;
		mt++;
	}
	plugin.format = types[mt];
	plugin.name = (char *)ModPlug_GetName(mpfile);
	plugin.artist = "";
	plugin.copyright = "";
	//plugin.format = "";
	plugin.length = ModPlug_GetLength(mpfile);
	plugin.subtunes = 1;
	plugin.tune = 0;

	if(plugin.length > 0)
		song_end = (plugin.length * 4) / 1000 * plugin.freq;
	else
		song_end = 999999999;
	total_samples = 0;

	char *d = (char *)data;

	//cprintf("%p %p %p %d %02x %02x\n", mpfile, plugin.name, data, size, d[0], d[1]);

	return 0;
}

static int close()
{
	if(mpfile)
		ModPlug_Unload(mpfile);
	mpfile = NULL;
	return 0;
}

static int fill_buffer(signed short *dest, int len)
{

	////if(total_samples > song_end)
	//	return 0;

	if(mpfile)
		ModPlug_Read(mpfile, dest, len);
	else
		memset(dest, 0, len);

	//total_samples += len;


	return len;
}

static int is_ext(const char *s, const char *ext)
{
	const char *sext = strrchr(s, '.');
	if(!sext)
		sext = "";
	return (stricmp(ext, sext) == 0);
}

static int can_handle(const char *name)
{
	return (is_ext(name, ".mod") || is_ext(name, ".xm") || is_ext(name, ".s3m") || is_ext(name, ".dbm") || is_ext(name, ".okt") || is_ext(name, ".far") || is_ext(name, ".ult") || is_ext(name, ".mt2") || is_ext(name, ".ptm") || is_ext(name, ".it") || is_ext(name, ".mdl"));
}

extern "C" {

#ifndef INIT_SOUND_PLUGIN
#define INIT_SOUND_PLUGIN modplay_init_sound_plugin
#endif

struct sound_plugin *INIT_SOUND_PLUGIN()
{
	memset(&plugin, 0, sizeof(plugin));

	plugin.plugname = "modplug";

	plugin.freq = 22050;
	plugin.channels = 2;
	//plugin.init_file = init_file;
	plugin.init_data = init_data;
	plugin.fill_buffer = fill_buffer;
	plugin.can_handle = can_handle;
	plugin.close = close;
	//plugin.set_position = set_position;

	return &plugin;
}

}
