/*  Copyright 2006 Jonas Minnberg

    This file is part of OldPlay - a portable, multiformat musicplayer.

    OldPlay is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OldPlay is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with OldPlay; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include <string.h>
#include <stdlib.h>

#include "Nsf_Emu.h"
#include "Gbs_Emu.h"
#include "Spc_Emu.h"
#include "Vgm_Emu.h"
#include "Gym_Emu.h"

#include "../plugin.h"

#ifndef WIN32
#define stricmp strcasecmp
#endif

static struct sound_plugin plugin;

static int is_ext(const char *s, const char *ext)
{
	const char *sext = strrchr(s, '.');
	if(!sext)
		sext = "";
	return (stricmp(ext, sext) == 0);
}

//static int init_data(char *name, void *data, int size)
//}

Music_Emu* emu; 
int playing = 0;
static Nsf_Emu::header_t nsf_header;
static Gbs_Emu::header_t gb_header;
static Spc_Emu::header_t spc_header;
static Vgm_Emu::header_t vg_header;
static Gym_Emu::header_t gy_header;

static int init_file(char *fname)
{
	//Gbs_Emu

	plugin.tune = 0;
	plugin.length = -1;
	plugin.freq = 22050;

	plugin.clockfreq = 150;


	if(is_ext(fname, ".nsf") )
	{
		Nsf_Emu *nemu = new Nsf_Emu();
		nemu->init(plugin.freq);
		// Load file
		Emu_Std_Reader reader;
		reader.open(fname);
		reader.read(&nsf_header, sizeof(nsf_header));
		nemu->load(nsf_header, reader);
		plugin.name = (char*)(nsf_header.game ? nsf_header.game : "");
		plugin.artist = (char*)(nsf_header.author ? (char*) nsf_header.author : "");
		plugin.copyright = (char*)(nsf_header.copyright ? (char*) nsf_header.copyright : "");
		plugin.format = "Nintendo 8Bit";
		plugin.tune = nsf_header.first_track-1;
		emu = nemu;
	}
	else
	if(is_ext(fname, ".gbs") )
	{
		Gbs_Emu *gbemu = new Gbs_Emu();
		gbemu->init(plugin.freq);
		// Load file
		Emu_Std_Reader reader;
		reader.open(fname);
		reader.read(&gb_header, sizeof(gb_header));
		gbemu->load(gb_header, reader);
		plugin.name = (char*)(gb_header.game ? (char*) gb_header.game : "");
		plugin.artist = (char*)(gb_header.author ? (char*) gb_header.author : "");
		plugin.copyright = (char*)(gb_header.copyright ? (char*) gb_header.copyright : "");
		plugin.format = "Gameboy";
		plugin.tune = nsf_header.first_track;
		emu = gbemu;
	}
	else
	if(is_ext(fname, ".spc") )
	{
		Spc_Emu *spcemu = new Spc_Emu();
		plugin.freq = 32000;
		spcemu->init(plugin.freq);
		// Load file
		Emu_Std_Reader reader;
		reader.open(fname);
		reader.read(&spc_header, sizeof(spc_header));
		spcemu->load(spc_header, reader);
		plugin.name = (char*)(spc_header.game ? (char*) spc_header.game : "");
		plugin.artist = (char*)(spc_header.author ? (char*) spc_header.author : "");
		plugin.copyright = (char*)(spc_header.copyright ? (char*) spc_header.copyright : "");
		plugin.format = "Super Nintendo";
		plugin.tune = nsf_header.first_track-1;
		spc_header.len_secs[3] = 0;
		plugin.length = atoi(spc_header.len_secs) * 1000;
		emu = spcemu;
	}
	else
	if(is_ext(fname, ".vgm") )
	{
		Vgm_Emu *vgemu = new Vgm_Emu();
		vgemu->init(plugin.freq);
		// Load file
		Emu_Std_Reader reader;
		reader.open(fname);
		reader.read(&vg_header, sizeof(vg_header));
		vgemu->load(vg_header, reader);
		plugin.name = (char*)(vg_header.game ? (char*) vg_header.game : "");
		plugin.artist = (char*)(vg_header.author ? (char*) vg_header.author : "");
		plugin.copyright = (char*)(vg_header.copyright ? (char*) vg_header.copyright : "");
		plugin.format = "Sega Master System";
		emu = vgemu;
	}
	else
	if(is_ext(fname, ".gym") )
	{
		Gym_Emu *gyemu = new Gym_Emu();
		gyemu->init(plugin.freq);
		// Load file
		Emu_Std_Reader reader;
		reader.open(fname);
		reader.read(&gy_header, sizeof(gy_header));
		gyemu->load(gy_header, reader);
		plugin.name = (char*)(gy_header.game ? (char*) gy_header.game : "");
		plugin.artist = (char*)(gy_header.author ? (char*) gy_header.author : "");
		plugin.copyright = (char*)(gy_header.copyright ? (char*) gy_header.copyright : "");
		plugin.format = "Sega Megadrive";
		emu = gyemu;
	}

	plugin.subtunes = emu->track_count();

	
	// Print game and song info
	//printf( "Game     : %-32s\n", header.game      ? (char*) header.game : "" );
	//printf( "Song     : %-32s\n", header.song      ? (char*) header.song : "" );
	//printf( "Author   : %-32s\n", header.author    ? (char*) header.author : "" );
	//printf( "Copyright: %-32s\n", header.copyright ? (char*) header.copyright : "" );
	//printf( "Tracks   : %d\n", emu->track_count() );
	//printf( "\n" );

	emu->start_track(plugin.tune);
	playing = 1;


	return 0;
}


static int fill_buffer(signed short *dest, int len)
{
	if(playing)
		emu->play(len/2, dest);
	else
		memset(dest, 0, len);

	return len;
}

static int set_position(int seconds, int subtune)
{
	emu->start_track(subtune);
	return 0;
}

static int can_handle(const char *name)
{
	return (is_ext(name, ".nsf") || is_ext(name, ".gbs") || is_ext(name, ".spc") || is_ext(name, ".gym") || is_ext(name, ".vgm"));
}

extern "C" {

#ifndef INIT_SOUND_PLUGIN
#define INIT_SOUND_PLUGIN gme_init_sound_plugin
#endif

struct sound_plugin *INIT_SOUND_PLUGIN()
{
	memset(&plugin, 0, sizeof(plugin));

	plugin.plugname = "gme";

	plugin.freq = 22050;
	plugin.channels = 2;
	plugin.init_file = init_file;
	//plugin.init_data = init_data;
	plugin.fill_buffer = fill_buffer;
	plugin.set_position = set_position;
	plugin.can_handle = can_handle;
	plugin.plugname = "GME";

	return &plugin;
}

}

