/*  Copyright 2006 Jonas Minnberg

    This file is part of OldPlay - a portable, multiformat musicplayer.

    OldPlay is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OldPlay is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with OldPlay; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <SDL.h>
#include "SDLScreen.h"
#include "Input.h"
#include "PlayerApp.h"
#include "State.h"

PlayerApp *app;

static int bcount = 0;
extern "C"
{
	void busy(int x)
	{
		if(x)
			bcount = (bcount+1) % 6;
		else
			bcount = 0;
		app->lcd->SetLed(LedControl::BUSY, bcount);
		app->lcd->Render();
		//SDL_Delay(1);
	}
}

extern BrowserState g_browserState;
extern PlaylistState g_playlistState;

#ifdef GP2X

#include <sys/times.h>

void hw_set_screen(bool on);
void hw_init();
void hw_set_cpu(int speed);
int hw_get_voltage();
#else
void hw_set_screen(bool on)
{
}
#endif


int current_clockfreq = -1;

int main(int argc, char **argv)
{
#ifdef GP2X
	struct tms buf;
	clock_t startt = times(&buf);
	fprintf(stderr, "Start %d\n", startt);

	hw_init();
#endif


	app = new PlayerApp(argc > 1 ? argv[1] : NULL);
	app->Layout();

	fprintf(stderr, "First Layout\n");

	g_screen.Update();

	Input *input = new Input();

	IState *currentState = &g_browserState;
	currentState->Enter();

	int display_on = 1;
	int input_hold = 0;
	int volt = -1;
	int count = 0;

	bool go_on = true;

	app->lcd->SetLed(LedControl::VOLT, 12);

	fprintf(stderr, "Entering mainloop\n");

#ifdef GP2X
	if(app->cpu_scaling)
	{
		current_clockfreq = 100;
		hw_set_cpu(100);
	}
#endif

	while(go_on)
	{
		SDL_Delay(60);

		app->Update();
		currentState->Update();

		if(display_on)
			app->Render();

		int key = input->GetKey(60);

#ifdef GP2X
		if(count++ == 50)
		{
			int v = hw_get_voltage();

			if(volt == -1)
				volt = v;
			else
				volt = (volt * 15 + v)>>4;

			app->lcd->SetLed(LedControl::VOLT, (v-18)*2/3);
			if(current_clockfreq > 0)
				app->lcd->SetLed(LedControl::EQ1, (current_clockfreq)/50);

			count = 0;
			
			//clock_t now = times(&buf);
			//fprintf(stderr, "%d %d %d\n",now, buf.tms_utime, buf.tms_stime);
			//fflush(stderr);


		}
#endif

		if(key == Input::HOLD)
		{
			input_hold ^= 1;
			app->lcd->SetLed(LedControl::HOLD, input_hold);
		}

		if(!input_hold && key)
		{
			if(go_on = app->HandleKey(key))
			{
				if(key == Input::SELECT)
				{
					if(currentState == &g_browserState)
						currentState = &g_playlistState;
					else
						currentState = &g_browserState;
					currentState->Enter();
				}
				else
					currentState->HandleKey(key);

				if(key == Input::BLANK)
				{
					display_on ^= 1;
					hw_set_screen((bool)display_on);
				}
#ifdef GP2X
				else
				if(key == Input::SPEED)
				{
					if(current_clockfreq < 0)
						current_clockfreq = 175;
					current_clockfreq += 25;
					if(current_clockfreq == 275)
						current_clockfreq = 50;
					hw_set_cpu(current_clockfreq);
					app->lcd->SetLed(LedControl::EQ1, (current_clockfreq)/50);
				}
#endif
			}
		}
	}

#ifdef GP2X
	hw_set_cpu(200);
#endif

	if(!display_on)
		hw_set_screen((bool)display_on);

	delete app;

	return 0;
}
