/*  Copyright 2006 Jonas Minnberg

    This file is part of OldPlay - a portable, multiformat musicplayer.

    OldPlay is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OldPlay is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with OldPlay; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include "Music_Emu.h"
#include "Gme_File.h"

#include "../plugin.h"
#include "../util.h"

static struct sound_plugin plugin;
static Music_Emu* emu;
static gme_err_t error;
static track_info_t* t_info = (track_info_t*)malloc(sizeof(track_info_t));
static int playing;

static int set_position(int seconds, int subtune);

static int init_file(char *fname)
{
    playing = 0;
    plugin.tune = 0;
    plugin.clockfreq = 150; 
    if (is_ext(fname, ".gym") || is_ext(fname, ".vgz") || is_ext(fname, ".vgm"))
        plugin.clockfreq = 200;

    if (error = gme_open_file( fname, &emu , plugin.freq))
    {
        printf("%s\n",error);
        return -1;
    }
    
    gme_set_stereo_depth( emu, 0.8 );
    
    ////// M3u //////
    char* fnamem3u = (char*)malloc(sizeof(char)*(strlen(fname)+1));
    strcpy(fnamem3u,fname);
    char* end = strrchr(fnamem3u,'.');
    strcpy(end,".m3u");
    error = gme_load_m3u( emu, (const char*)fnamem3u );
    free(fnamem3u);

    playing = 1;
    set_position(0,plugin.tune);
    return 0;
}

static int close()
{
    gme_delete(emu);
    plugin.artist = plugin.name = plugin.subname = plugin.copyright = plugin.format = NULL;
    plugin.tune = plugin.subtunes = plugin.length = 0;
    return 0;
}

static int fill_buffer(signed short *dest, int len)
{
    if(playing)
    {
        gme_play(emu, len>>1, dest);
        if(gme_track_ended( emu ))
            plugin.length = gme_tell( emu );
    }
    else
        memset(dest, 0, len);
    return len;
}

//void hw_set_cpu(int speed); // For CPU-boost
static int set_position(int seconds, int subtune)
{
    if (!playing)
        return 0;
    if (!seconds)
    {
        gme_start_track(emu, subtune);
        plugin.tune = subtune;
        if (!(error = gme_track_info( emu, t_info, subtune ))) {
            if (t_info->length > 0)
                plugin.length = t_info->length;
            else if (t_info->intro_length >= 0 && t_info->loop_length > 0)
                plugin.length = t_info->intro_length + t_info->loop_length * 2;
            else
                plugin.length = 150000; // 2:30
            plugin.artist    = t_info->author;
            plugin.name      = t_info->game;
            plugin.subname   = t_info->song;
            plugin.copyright = t_info->copyright;
            plugin.subtunes  = t_info->track_count;
            plugin.format    = t_info->system;
            gme_set_fade(emu, plugin.length-3000);
            return subtune;
        }
    }
    else
    {
        //hw_set_cpu(225);
        // Seek 10 seconds
        if (error = emu->seek(emu->tell()+(long)seconds*10*1000))
            printf("%s\n",error);
        return seconds*10;
        //hw_set_cpu(plugin.clockfreq);
    }
}

static int can_handle(const char *name)
{
    return (is_ext(name, ".nsf") || is_ext(name, ".nsfe") || 
            is_ext(name, ".spc") || is_ext(name, ".gym") || 
            is_ext(name, ".vgm") || is_ext(name, ".vgz") ||
            is_ext(name, ".sap") || is_ext(name, ".hes") ||
            is_ext(name, ".kss") || is_ext(name, ".gbs") ||
            is_ext(name, ".ay"));
}

extern "C" {

#ifndef INIT_SOUND_PLUGIN
#define INIT_SOUND_PLUGIN gme_init_sound_plugin
#endif

struct sound_plugin *INIT_SOUND_PLUGIN()
{
    memset(&plugin, 0, sizeof(plugin));
    plugin.plugname = "GME";
    //plugin.init_data = init_data;
    plugin.init_file = init_file;
    //plugin.request_format = NULL;
    plugin.set_position = set_position;
    plugin.fill_buffer = fill_buffer;
    plugin.can_handle = can_handle;
    plugin.close = close;
    plugin.channels = 2;
    plugin.freq = 44100;//22050;
    return &plugin;
}

}

