/*  Copyright 2006 Jonas Minnberg

    This file is part of OldPlay - a portable, multiformat musicplayer.

    OldPlay is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    OldPlay is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with OldPlay; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <SDL.h>
#include <string.h>

#include "MusicPlayer.h"
#include "PlayerApp.h"
#include "zip/zipint.h"

#ifndef _WIN32
#define stricmp strcasecmp
#define strnicmp strncasecmp
#include <unistd.h>
#else
#include <direct.h>
#endif

using namespace std;

int is_ext(const char *s, const char *ext)
{
	const char *sext = strrchr(s, '.');
    return sext && stricmp(ext, sext) == 0;
}

int compare(const void* a, const void* b)
{
    return strcasecmp((const char *)a, (const char *)b);
}

int read_file(char *fbuf, const char* fname, int fsize)
{
    FILE* fp = fopen(fname,"rb"); 
    if (fp)
    {
        int fsize = fread(fbuf, sizeof(char), fsize, fp);
        fclose(fp);
        return fsize;
    }
    else
        return -1;
}

int read_zip(char *dest, const char* zipname, int length, int index)
{
    struct zip *z = zip_open(zipname,0,NULL);
    zip_file *zfp = zip_fopen_index(z, index, 0);
    int len = zip_fread(zfp, dest, length);
    zip_fclose(zfp);
    fclose(z->zp);
    return len;
}

int plugin_filter_cb(void *data, const char *name)
{
	PlayerApp *app = (PlayerApp *)data;
	if(app->music_player->CanHandle(name))
		return 1;
	if(is_ext(name, ".zip") || is_ext(name, ".m3u") || is_ext(name, ".pls") | is_ext(name, ".opl"))
		return 1;
	return 0;
}

int plugin_filter_zip_cb(void *data, const char *name)
{
    if (is_ext(name,".zip"))
        return 0;
    return plugin_filter_cb(data, name);
}

int filter_opl_cb(void *data, const char *name)
{
    return is_ext(name,".opl");
}
